"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FailedGitReport = exports.GitReport = void 0;
const tslib_1 = require("tslib");
const faker_1 = tslib_1.__importDefault(require("faker"));
const os_1 = require("os");
const colorette_1 = require("colorette");
class GitReport {
    constructor({ committers, weeks, projects }) {
        const totals = GitReport.computeTotals(committers);
        this.committers = GitReport.sortCommittersByTotalCommitsDesc(committers);
        this.weeks = weeks;
        this.projects = projects;
        this.totalCommits = totals.totalCommits;
        this.totalDeletions = totals.totalDeletions;
        this.totalFilesChanged = totals.totalFilesChanged;
        this.totalInsertions = totals.totalInsertions;
    }
    anonymize() {
        return new GitReport({
            weeks: this.weeks,
            projects: this.projects,
            committers: this.committers.map(committer => ({
                ...committer,
                email: faker_1.default.internet.email(),
                name: faker_1.default.name.findName(),
            })),
        });
    }
    hasFailed() {
        return false;
    }
    toString() {
        if (this.projects.length === 0) {
            return colorette_1.yellow('⚠️  There is no git projects to report.');
        }
        return `Report for:
${this.projects.map(project => `  - ${project}`).join(os_1.EOL)}

Total commits in the last ${this.weeks} weeks: ${this.totalCommits}${this.committers.length > 0 ? `${os_1.EOL}Contributions by author:${os_1.EOL}` : ''}${this.committers.map(({ name, email, totalCommits, totalFilesChanged, totalInsertions, totalDeletions, }) => `${os_1.EOL}    ${name} (${email}):
      Commits: ${totalCommits}
      Files changed: ${totalFilesChanged}
      Insertions: ${totalInsertions}
      Deletions: ${totalDeletions}`).join(os_1.EOL)}`;
    }
    static sortCommittersByTotalCommitsDesc(committers) {
        return [...committers].sort((a, b) => {
            return GitReport.computeContributionScore(b) - GitReport.computeContributionScore(a) || a.name.localeCompare(b.name);
        });
    }
    static computeTotals(committers) {
        return committers.reduce((totals, committerInfo) => ({
            totalCommits: totals.totalCommits + committerInfo.totalCommits,
            totalDeletions: totals.totalDeletions + committerInfo.totalDeletions,
            totalFilesChanged: totals.totalFilesChanged + committerInfo.totalFilesChanged,
            totalInsertions: totals.totalInsertions + committerInfo.totalInsertions,
        }), {
            totalCommits: 0,
            totalDeletions: 0,
            totalFilesChanged: 0,
            totalInsertions: 0,
        });
    }
    static computeContributionScore({ totalCommits, totalFilesChanged, totalInsertions, totalDeletions }) {
        return totalCommits + totalFilesChanged + totalInsertions + totalDeletions;
    }
}
exports.GitReport = GitReport;
class FailedGitReport extends GitReport {
    constructor(projects) {
        super({
            projects,
            weeks: 0,
            committers: [],
        });
    }
    hasFailed() {
        return true;
    }
}
exports.FailedGitReport = FailedGitReport;
//# sourceMappingURL=git-report.js.map