"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitReportMapper = void 0;
const tslib_1 = require("tslib");
const os_1 = require("os");
const path_1 = tslib_1.__importDefault(require("path"));
const git_report_1 = require("../domain/git-report");
class GitReportMapper {
    static toDomain(gitLog, weeks, projectPath) {
        if (gitLog === undefined) {
            return new git_report_1.FailedGitReport([GitReportMapper.extractProjectName(projectPath)]);
        }
        const contributors = gitLog.split(os_1.EOL.concat(os_1.EOL)).filter(Boolean);
        return new git_report_1.GitReport({
            weeks,
            projects: [GitReportMapper.extractProjectName(projectPath)],
            committers: GitReportMapper.extractCommittersInfo(contributors),
        });
    }
    static extractProjectName(projectPath) {
        const absolutePath = path_1.default.resolve(projectPath);
        return absolutePath.slice(absolutePath.lastIndexOf('/') + 1);
    }
    static extractCommittersInfo(gitLogStats) {
        const contributorsStats = GitReportMapper.extractContributorsStats(gitLogStats);
        return Object.entries(contributorsStats).map(GitReportMapper.generateCommitterInfo);
    }
    static generateCommitterInfo([contributor, { totalCommits, stats }]) {
        return ({
            ...GitReportMapper.extractContributorInfo(contributor),
            ...GitReportMapper.extractStats(stats),
            totalCommits,
        });
    }
    static extractContributorsStats(gitLogStats) {
        return gitLogStats.reduce((contributorsStats, gitLogStat) => {
            const [contributor, stats] = gitLogStat.split(os_1.EOL);
            return {
                ...contributorsStats,
                [contributor]: {
                    stats: [...(contributorsStats[contributor]?.stats || []), stats],
                    totalCommits: (contributorsStats[contributor]?.totalCommits || 0) + 1,
                },
            };
        }, {});
    }
    static extractContributorInfo(contributor) {
        const [name, email] = contributor.split('|');
        return {
            name: String(name).trim(),
            email: String(email).trim(),
        };
    }
    static extractStats(stats) {
        return stats.reduce((acc, stat) => {
            const filesChanged = GitReportMapper.parseStat('files? changed', stat);
            const insertions = GitReportMapper.parseStat('insertions?', stat);
            const deletions = GitReportMapper.parseStat('deletions?', stat);
            return {
                totalFilesChanged: filesChanged + acc.totalFilesChanged,
                totalInsertions: insertions + acc.totalInsertions,
                totalDeletions: deletions + acc.totalDeletions,
            };
        }, {
            totalFilesChanged: 0,
            totalInsertions: 0,
            totalDeletions: 0,
        });
    }
    static parseStat(regExp, stat) {
        const extractedStat = new RegExp(`\\d+ ${regExp}`).exec(stat);
        if (extractedStat) {
            const stat = /\d+/.exec(extractedStat[0]);
            return Number(stat[0]);
        }
        return 0;
    }
}
exports.GitReportMapper = GitReportMapper;
//# sourceMappingURL=git-report.mapper.js.map